#include "MLibsDll.h"
#include "../MString.h"
#include "../MAnsiString.h"

errno_t strcpy_s(char *dst, size_t num, const char *src) {
	if(dst == nullptr) return EINVAL;
	if(src == nullptr) return EINVAL;
	if(num == 0) return ERANGE;
	if(strlen(src) >= num){
		strncpy(dst, src, num-1);
		return ERANGE;
	}
	strcpy(dst, src);
	return 0;
}
errno_t memcpy_s(void *dst, size_t nelems, const void *src, size_t n) {
	if(dst == nullptr) return EINVAL;
	if(src == nullptr){
		if(nelems) memset(dst, 0, nelems);
		return EINVAL;
	}
	if(nelems < n){
		if(nelems) memset(dst, 0, nelems);
		return ERANGE;
	}
	memcpy(dst, src, n);
	return 0;
}
int _wtoi(const wchar_t *s) {
	try {
		return std::stoi(std::wstring(s));
	}catch(...){
		return 0;
	}
}
double _wtof(const wchar_t *s) {
	try {
		return std::stod(std::wstring(s));
	}catch(...){
		return 0.0;
	}
}
int64_t _atoi64(const char *s) {
	try {
		return std::stoll(std::string(s));
	}catch(...){
		return 0;
	}
}
int64_t _wtoi64(const wchar_t *s) {
	try {
		return std::stoll(std::wstring(s));
	}catch(...){
		return 0;
	}
}
int64_t _strtoi64(const char *s, size_t *idx, int base) {
	try {
		return std::stoll(std::string(s), idx, base);
	}catch(...){
		return 0;
	}
}
uint64_t _strtoui64(const char *s, size_t *idx, int base) {
	try {
		return std::stoull(std::string(s), idx, base);
	}catch(...){
		return 0;
	}
}
int64_t _wcstoi64(const wchar_t *s, size_t *idx, int base) {
	try {
		return std::stoll(std::wstring(s), idx, base);
	}catch(...){
		return 0;
	}
}
int64_t _wcstoui64(const wchar_t *s, size_t *idx, int base) {
	try {
		return std::stoull(std::wstring(s), idx, base);
	}catch(...){
		return 0;
	}
}

errno_t fopen_s(FILE** fh, const char *filename, const char *mode)
{
	if(fh == nullptr || filename == nullptr || mode == nullptr)
		return EINVAL;
	*fh = fopen(filename, mode);
	if(*fh == nullptr) return EINVAL;
	return 0;
}

errno_t _wfopen_s(FILE** fh, const wchar_t *filename, const wchar_t *mode)
{
	if(fh == nullptr || filename == nullptr || mode == nullptr)
		return EINVAL;
	*fh = fopen(MString(filename).toUtf8String().c_str(), MString(mode).toUtf8String().c_str());
	if(*fh == nullptr) return errno;
	return 0;
}

BOOL IsDBCSLeadByte(BYTE ch)
{
	// Use Shift-JIS
	if(ch >= 128 && ch <= 160) return 1;
	if(ch >= 224 && ch <= 255) return 1;
	return 0;
}


