﻿//---------------------------------------------------------------------------
//
//   StationSpy
//
//          Copyright(C) 2006-2019, tetraface Inc.
//
//     This plug-in watches messages sent to a station plug-in. The sent 
//    messages are displayed in the list. Please use to confirm what and 
//    when messages are sent.
//
//    　常駐プラグインに送られるメッセージを監視します。送られたメッセー
//    ジはリストに一覧表示されます。
//    　常駐プラグインにどういったタイミングでメッセージが送られるかを確
//    認する目的でご利用ください。
//
//---------------------------------------------------------------------------
#include "stdafx.h"
#include "maindlg.h"
#include "StationSpyPlugin.h"

static StationSpyPlugin plugin;


// Constructor
// コンストラクタ
StationSpyPlugin::StationSpyPlugin()
{
	dlgMain = NULL;
	m_bActivate = false;
	m_bOnDraw = false;
	m_bOnUpdateScene = false;
	m_bOnUpdateUndo = false;
	m_bWidgetEvents = false;
	m_bMouseMove = false;
}

// Destructor
// デストラクタ
StationSpyPlugin::~StationSpyPlugin()
{
	if (dlgMain != NULL){
		// Also deletes the window here just to make sure though it has been 
		// deleted in Exit().
		// Exit()で破棄されるはずだが一応念のためここでも
		delete dlgMain;
		dlgMain = NULL;
	}
}

//---------------------------------------------------------------------------
//  GetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
void StationSpyPlugin::GetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x9CE001E3;
}

//---------------------------------------------------------------------------
//  GetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
const char *StationSpyPlugin::GetPlugInName(void)
{
	return "Station Spy       Copyright(C) 2006-2019, tetraface Inc.";
}

//---------------------------------------------------------------------------
//  EnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
const wchar_t *StationSpyPlugin::EnumString(void)
{
	return L"Station Spy";
}


//---------------------------------------------------------------------------
//  EnumSubCommand
//    サブコマンド前を列挙
//---------------------------------------------------------------------------
const char *StationSpyPlugin::EnumSubCommand(int index)
{
	switch(index){
	case 0: return "Message";
	case 1: return "Vertex";
	case 2: return "Face";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  GetSubCommandString
//    サブコマンドの文字列を列挙
//---------------------------------------------------------------------------
const wchar_t *StationSpyPlugin::GetSubCommandString(int index)
{
	switch(index){
	case 0: return L"Message";
	case 1: return L"Vertex";
	case 2: return L"Face";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  Initialize
//    アプリケーションの初期化
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::Initialize()
{
	if (dlgMain == NULL){
		// Create a new window that's parent is the main window.
		// メインウインドウを親にした新しいウィンドウの作成
		MQWindowBase mainwnd = MQWindow::GetMainWindow();
		dlgMain = new CMainDlg(this, mainwnd);

		dlgMain->AddMessage(L"Initialize");
	}
	return TRUE;
}

//---------------------------------------------------------------------------
//  Exit
//    アプリケーションの終了
//---------------------------------------------------------------------------
void StationSpyPlugin::Exit()
{
	if (dlgMain != NULL){
		dlgMain->AddMessage(L"Exit");
		delete dlgMain;
		dlgMain = NULL;
	}
}

//---------------------------------------------------------------------------
//  Activate
//    表示・非表示切り替え要求
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::Activate(MQDocument doc, BOOL flag)
{
	if (dlgMain == NULL)
		return FALSE;

	// Switch showing and hiding the window.
	// ウインドウの表示・非表示切り替え
	m_bActivate = flag ? true : false;
	dlgMain->SetVisible(m_bActivate);

	if(m_bActivate){
		dlgMain->UpdateList(doc, false);
	}

	wchar_t buf[256];
	swprintf_s(buf, L"Activate : flag=%s", flag ? L"TRUE" : L"FALSE");
	dlgMain->AddMessage(buf);

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  IsActivated
//    表示・非表示状態の返答
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::IsActivated(MQDocument doc)
{
	// Return TRUE if a window has been not created yet.
	// ウインドウがまだ生成されていないならTRUE
	if (dlgMain == NULL)
		return FALSE;

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  OnMinimize
//    ウインドウの最小化への返答
//---------------------------------------------------------------------------
void StationSpyPlugin::OnMinimize(MQDocument doc, BOOL flag)
{
	wchar_t buf[256];
	swprintf_s(buf, L"OnMinimize : flag=%s", flag ? L"TRUE" : L"FALSE");
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnReceiveUserMessage
//    プラグイン独自のメッセージの受け取り
//---------------------------------------------------------------------------
int StationSpyPlugin::OnReceiveUserMessage(MQDocument doc, DWORD src_product, DWORD src_id, const char *description, void *message)
{
	size_t len = strlen(description);
	wchar_t *buf = new wchar_t[len + 200];
	swprintf_s(buf, len + 200, L"OnReceiveUserMessage : src_plugin=[%08x %08x] description=%S", src_product, src_id, description);
	dlgMain->AddMessage(buf);

	// Operate a special action only when the message was sent from SingleMove plug-in.
	// SingleMoveプラグインからの通知のみ、特別な動作を行う
	if (src_product == 0x56A31D20 && src_id == 0x3901EA9B){
		if (strcmp(description, "text") == 0){
			swprintf_s(buf, len + 200, L"  from SingleMove : %S", (const char*)message);
			dlgMain->AddMessage(buf);
		}
	}

	delete[] buf;

	// Return 59 without a deep idea.
	// 適当に59を返している
	return 59;
}

//---------------------------------------------------------------------------
//  OnSubCommand
//    A message for calling a sub comand
//    サブコマンドの呼び出し
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::OnSubCommand(MQDocument doc, int index)
{
	switch(index){
	case 0:
		dlgMain->m_Tab->SetCurrentPage(0);
		return FALSE;
	case 1:
		dlgMain->m_Tab->SetCurrentPage(1);
		return FALSE;
	case 2:
		dlgMain->m_Tab->SetCurrentPage(2);
		return FALSE;
	}
	return FALSE;
}

//---------------------------------------------------------------------------
//  OnDraw
//    描画時の処理
//---------------------------------------------------------------------------
void StationSpyPlugin::OnDraw(MQDocument doc, MQScene scene, int width, int height)
{
	if (!m_bOnDraw)
		return;

	wchar_t buf[256];
	swprintf_s(buf, L"OnDraw : width=%d, height=%d", width, height);
	dlgMain->AddMessage(buf);
}


//---------------------------------------------------------------------------
//  OnNewDocument
//    ドキュメント初期化時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnNewDocument(MQDocument doc, const char *filename, NEW_DOCUMENT_PARAM& param)
{
	wchar_t buf[256+MAX_PATH];
	swprintf_s(buf, L"OnNewDocument : filename=%S", (filename!=NULL) ? filename : "(none)");
	dlgMain->AddMessage(buf);

	if(m_bActivate){
		dlgMain->UpdateList(doc, false);
	}
}

//---------------------------------------------------------------------------
//  OnEndDocument
//    ドキュメント終了時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnEndDocument(MQDocument doc)
{
	dlgMain->AddMessage(L"OnEndDocument");
}

//---------------------------------------------------------------------------
//  OnSaveDocument
//    ドキュメント保存時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnSaveDocument(MQDocument doc, const char *filename, SAVE_DOCUMENT_PARAM& param)
{
	wchar_t buf[256+MAX_PATH];
	swprintf_s(buf, L"OnSaveDocument : filename=%S", (filename!=NULL) ? filename : "(none)");
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnUndo
//    アンドゥ実行時
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::OnUndo(MQDocument doc, int undo_state)
{
	wchar_t buf[256];
	swprintf_s(buf, L"OnUndo : undo_state=%d", undo_state);
	dlgMain->AddMessage(buf);

	if(m_bActivate){
		dlgMain->UpdateList(doc, false);
	}
	
	return FALSE;
}

//---------------------------------------------------------------------------
//  OnRedo
//    リドゥ実行時
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::OnRedo(MQDocument doc, int redo_state)
{
	wchar_t buf[256];
	swprintf_s(buf, L"OnRedo : redo_state=%d", redo_state);
	dlgMain->AddMessage(buf);

	if(m_bActivate){
		dlgMain->UpdateList(doc, false);
	}

	return FALSE;
}

//---------------------------------------------------------------------------
//  OnUpdateUndo
//    アンドゥ状態更新時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateUndo(MQDocument doc, int undo_state, int undo_size)
{
	if (!m_bOnUpdateUndo)
		return;

	wchar_t buf[256];
	swprintf_s(buf, L"OnUpdateUndo : undo_state=%d undo_size=%d", undo_state, undo_size);
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnObjectModified
//    オブジェクトの編集時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnObjectModified(MQDocument doc)
{
	dlgMain->AddMessage(L"OnObjectModified");

	if(m_bActivate){
		dlgMain->UpdateList(doc, false);
	}
}

//---------------------------------------------------------------------------
//  OnObjectSelected
//    オブジェクトの選択状態の変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnObjectSelected(MQDocument doc)
{
	dlgMain->AddMessage(L"OnObjectSelected");

	if(m_bActivate){
		dlgMain->UpdateList(doc, true);
	}
}

//---------------------------------------------------------------------------
//  OnUpdateObjectList
//    カレントオブジェクトの変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateObjectList(MQDocument doc)
{
	dlgMain->AddMessage(L"OnUpdateObjectList");

	if(m_bActivate){
		dlgMain->UpdateList(doc, false);
	}
}

//---------------------------------------------------------------------------
//  OnMaterialModified
//    マテリアルのパラメータ変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnMaterialModified(MQDocument doc)
{
	dlgMain->AddMessage(L"OnMaterialModified");
}

//---------------------------------------------------------------------------
//  OnUpdateMaterialList
//    カレントマテリアルの変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateMaterialList(MQDocument doc)
{
	dlgMain->AddMessage(L"OnUpdateMaterialList");
}

//---------------------------------------------------------------------------
//  OnUpdateScene
//    シーン情報の変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateScene(MQDocument doc, MQScene scene)
{
	if (!m_bOnUpdateScene)
		return;

	dlgMain->AddMessage(L"OnUpdateScene");
}

//---------------------------------------------------------------------------
//  ExecuteCallback
//    コールバックに対する実装部
//---------------------------------------------------------------------------
bool StationSpyPlugin::ExecuteCallback(MQDocument doc, void *option)
{
	CallbackInfo *info = (CallbackInfo*)option;
	return ((*this).*info->proc)(doc);
}

// コールバックの呼び出し
void StationSpyPlugin::Execute(ExecuteCallbackProc proc)
{
	CallbackInfo info;
	info.proc = proc;
	BeginCallback(&info);
}

// プラグインのベースクラスを返す
MQBasePlugin *GetPluginClass()
{
	return &plugin;
}

